@@ -6,7 +6,7 @@
 """
 import datetime
 
-def xml_to_dict(root_or_str):
+def xml_to_dict(root_or_str, strict=True):
     """
     Converts `root_or_str` which can be parsed xml or a xml string to dict.
 
@@ -15,7 +15,7 @@ def xml_to_dict(root_or_str):
     if isinstance(root, str):
         import xml.etree.cElementTree as ElementTree
         root = ElementTree.XML(root_or_str)
-    return {root.tag: _from_xml(root)}
+    return {root.tag: _from_xml(root, strict)}
 
 def dict_to_xml(dict_xml):
     """
@@ -38,17 +38,38 @@ def _to_xml(el):
         val = str(el).lower()
     else:
         val = el
+    if val is None: val = 'null'
     return val
 
+def _extract_attrs(els):
+    """
+    Extracts attributes from dictionary `els`. Attributes are keys which start
+    with '@'
+    """
+    if not isinstance(els, dict):
+        return ''
+    return ''.join(' %s="%s"' % (key[1:], value) for key, value in els.iteritems()
+                   if key.startswith('@'))
+
 def _dict_to_xml(els):
     """
     Converts `els` which is a python dict to corresponding xml.
     """
+    def process_content(tag, content):
+        attrs = _extract_attrs(content)
+        text = isinstance(content, dict) and content.get('#text', '') or ''
+        return '<%s%s>%s%s</%s>' % (tag, attrs, _to_xml(content), text, tag)
+
     tags = []
     for tag, content in els.iteritems():
-        if isinstance(content, list):
+        # Text and attributes
+        if tag.startswith('@') or tag == '#text':
+            continue
+        elif isinstance(content, list):
             for el in content:
-                tags.append('<%s>%s</%s>' % (tag, _to_xml(el), tag))
+                tags.append(process_content(tag, el))
+        elif isinstance(content, dict):
+            tags.append(process_content(tag, content))
         else:
             tags.append('<%s>%s</%s>' % (tag, _to_xml(content), tag))
     return ''.join(tags)
@@ -83,7 +104,7 @@ def _str_to_boolean(bool_str):
         return True
     return False
 
-def _from_xml(el):
+def _from_xml(el, strict):
     """
     Extracts value of xml element element `el`.
     """
@@ -91,16 +112,23 @@ def _from_xml(el):
     # Parent node.
     if el:
         if _is_xml_el_dict(el):
-            val = _dict_from_xml(el)
+            val = _dict_from_xml(el, strict)
         elif _is_xml_el_list(el):
-            val = _list_from_xml(el)
+            val = _list_from_xml(el, strict)
     # Simple node.
     else:
         attribs = el.items()
-        # An element with no subelements but text.
-        if el.text:
-            val = _val_and_maybe_convert(el)
         # An element with attributes.
+        if attribs and strict:
+            val = dict(('@%s' % k, v) for k, v in dict(attribs).iteritems())
+            if el.text:
+                converted = _val_and_maybe_convert(el)
+                val['#text'] = el.text
+                if converted != el.text:
+                    val['#value'] = converted
+        elif el.text:
+            # An element with no subelements but text.
+            val = _val_and_maybe_convert(el)
         elif attribs:
             val = dict(attribs)
     return val
@@ -122,22 +150,22 @@ _val_and_maybe_convert.convertors = {
     'integer': int
 }
 
-def _list_from_xml(els):
+def _list_from_xml(els, strict):
     """
     Converts xml elements list `el_list` to a python list.
     """
     res = []
     tag = els[0].tag
     for el in els:
-        res.append(_from_xml(el))
+        res.append(_from_xml(el, strict))
     return {tag: res}
 
-def _dict_from_xml(els):
+def _dict_from_xml(els, strict):
     """
     Converts xml doc with root `root` to a python dict.
     """
     # An element with subelements.
     res = {}
     for el in els:
-        res[el.tag] = _from_xml(el)
+        res[el.tag] = _from_xml(el, strict)
     return res
